@@ -1,4 +1,4 @@
-. ./version.sh
+. ./version.sh || exit 1
 
 corpus_size=large
 
@@ -25,7 +25,7 @@ do
 		echo "error: unknown performance test option '$1'" >&2; exit 1 ;;
 	esac
 done
-. ../test/test-lib-common.sh
+. ../test/test-lib-common.sh || exit 1
 
 set -e
 
@@ -203,7 +203,7 @@ time_done ()
     fi
 }
 
-cd -P "$test" || error "Cannot setup test environment"
+cd -P "$test" || error "Cannot set up test environment"
 test_failure=0
 test_count=0
 
